<?php
/**
* Plugin CacheMe
*
* @package	PLX
* @version	1.0
* @date	20/10/24
* @author Bronco
**/
class CacheMe extends plxPlugin {
	private $current_file;
	private $duration;
	private $cache_content;
	private $avoid_caching;
	private $timer_start;
	
	public function __construct($default_lang) {
		# appel du constructeur de la classe plxPlugin (obligatoire)
		parent::__construct($default_lang);
		$url=$this->getURL();
		$this->avoid_caching=strpos($url,'?preview')!==false;
		$this->current_file=hash('sha256', $url);
		# limite l'acces a l'ecran de configuration du plugin
		# PROFIL_ADMIN , PROFIL_MANAGER , PROFIL_MODERATOR , PROFIL_EDITOR , PROFIL_WRITER
		$this->setConfigProfil(PROFIL_ADMIN);		
		$this->duration=$this->getParam("duration");
		$this->duration=empty($this->duration)?30:$this->duration;
		
		# Declaration d'un hook (existant ou nouveau)
		$this->addHook('IndexBegin','IndexBegin');
		$this->addHook('IndexEnd','IndexEnd');
		$this->addHook('AdminPrepend','AdminPrepend');

		

		
	}

	# Activation / desactivation
	public function OnActivate() {
		if (!is_dir(PLX_ROOT.'temp')){
			mkdir (PLX_ROOT.'temp',0777);
			return false;
		}
	}
	public function OnDeactivate() {
		# code à executer à la désactivation du plugin
	}
	

	########################################
	# HOOKS
	########################################


	########################################
	# IndexBegin
	########################################
	# Description:
	public function IndexBegin(){
		if ($this->avoid_caching){return;}

		if (!empty($_POST['clear_cache'])){exit('yes');}

		$this->timer_start=-microtime(true);	
		
		if (!empty($this->current_file)){
			# si le fichier n'est pas dans le cache, on démarre le tampon
			$contenu=$this->cacheRead();
			if (!$contenu){
				echo '<?php ob_start();?>'; 				
			}else{
				$time= sprintf('%f', $this->timer_start += microtime(true));
				exit("$contenu<div class=\"cache_timer\">Cache version: $time s</div>");
			}	
		}
		return;
	}
	public function AdminPrepend(){
		if (isset($_POST['cacheme_clear_cache'])){
			$this->cacheClear();
		}

	}

	########################################
	# IndexEnd
	########################################
	# Description:
	public function IndexEnd(){
		if ($this->avoid_caching){return;}
		# si on est là, c'est que la page n'était pas en cache (sinon on a exit le contenu avant)
		# comme le $output d'index est plus bas que l'appel du hook, il faut récrire le code de la page index
		# pour output puis cacher.
		echo '<?php

				if ($plxMotor->aConf["urlrewriting"]) {
					$output = plxUtils::rel2abs($plxMotor->aConf["racine"], $output);
				}

				# On applique la compression gzip si nécessaire et disponible
				if ($plxMotor->aConf["gzip"]) {
					if($encoding=plxUtils::httpEncoding()) {
						header("Content-Encoding: $encoding");
						$output = gzencode($output,-1,FORCE_GZIP);
					}
				}

				# Restitution écran
				echo $output;

				$_cache_data=ob_get_clean();
				$plxMotor->plxPlugins->aPlugins["CacheMe"]->endCache($_cache_data);
				

		?>';


		
	}

	private function getURL(){
        $url = @( $_SERVER["HTTPS"] != 'on' ) ? 'http://'.$_SERVER["SERVER_NAME"] : 'https://'.$_SERVER["SERVER_NAME"];
        $url .= $_SERVER["SCRIPT_NAME"].'?'.$_SERVER['QUERY_STRING'];
        return $url;
    } 

	private function cacheRead(){
		$file=PLX_ROOT.'temp/'.$this->current_file;
		if (file_exists($file)){
			$filedat=@filemtime($file);
			$current=intval(date('U'));
			if ($filedat>=@$current&&$filedat!=0 || $filedat==0){
			    #fichier valable ou éternel (0 pour la durée)
	    		return file_get_contents($file); 
			}elseif ($filedat<$current&&$filedat!=0){
				#fichier obsolète
				if (file_exists($file)){
					unlink($file);
				}
				return false;
			}
		}else{
			return false;
		}
	}

	private function cacheWrite(){
		$file=PLX_ROOT.'temp/'.$this->current_file;
		file_put_contents($file,$this->cache_content);
		chmod($file,0777);
		if ($this->duration!=0){$this->duration=intval(@date('U'))+(60*$this->duration);}
		touch($file,$this->duration);
	}

	private function cacheClear(){
	  if (!is_dir(PLX_ROOT.'temp/')){mkdir ('./temp');return false;}//crée le /temp/ si nécessaire
	  // vide le cache
	  $fs=glob(PLX_ROOT.'temp/*');
	  foreach ($fs as $file){
	    unlink ($file);
	  }
	}

	public function endCache($data){
		if (!empty($this->current_file)){
			$this->cache_content=$data;
  			$this->cacheWrite();	
  			echo $this->cache_content;

  			$time= sprintf('%f', $this->timer_start += microtime(true));
				exit("<div class=\"cache_timer\">Caching time $time s</div>");
		}
	}
}





/* Pense-bete:
 * Récuperer des etres du fichier eters.xml
 *	$this->getParam("<nom du etre>")
 *	$this-> setParam ("1", 12345, "numeric")
 *	$this->saveParams()
 *
 *	plxUtils::strCheck($string) : sanitize string
 * Accéder à une méthode de plugin depuis un hook sur une page admin
 *  echo '<?php $plxAdmin->plxPlugins->aPlugins["NOMDUPLUGIN"]->METHODE()?>';
 * ou 
 *  echo '<?php $this->plxPlugins->aPlugins["NOMDUPLUGIN"]->METHODE()?>';
 * 
 * Quelques constantes utiles: 
 * PLX_CORE
 * PLX_ROOT
 * PLX_CHARSET
 * PLX_PLUGINS
 * PLX_CONFIG_PATH
 * PLX_ADMIN (true si on est dans admin)
 * PLX_CHARSET
 * PLX_VERSION
 * PLX_FEED
 *
 * Appel de HOOK dans un thème
 *	eval($plxShow->callHook("ThemeEndHead","1"))  ou eval($plxShow->callHook("ThemeEndHead",array("1","2")))
 *	ou $retour=$plxShow->callHook("ThemeEndHead","1"));
 */